/**
 *	@file
 *		AspinaCommands.h
 *	@brief
		Plexmotion Standard Link Interface Library
 *	@attention
 *		Copyright (C) 2021 - SHINANO-KENSHI Co,Ltd
 */
/*
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   * Neither the name, the copyright and the trademark of the SHINANO-KENSHI
 *     CO.,LTD. nor the names of its contributors may be used to endorse or
 *     promote products derived from this software without specific prior
 *     written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <ros/ros.h>
#include <std_msgs/String.h>
#include "param.h"
/**
 * publish用文字列
 */
#define ASPINA_GET_VERSION					"%d070101"			// ID:1 バージョン取得

#define ASPINA_GET_CURRENT_TORQUE			"%d07012e"			// ID:46(0x2e) torque取得
#define ASPINA_GET_CURRENT_SPEED			"%d07012f"			// ID:47(0x2f) speed取得
#define ASPINA_GET_CURRENT_ALARM			"%d070150"			// ID:80(0x50) alarm取得

#define ASPINA_SET_SERVO_ONOFF				"%d06021e%02x"		// ID:30(0x1e) サーボオンオフ切り替え 引数1(0:サーボオフ 1:サーボオン)
#define ASPINA_GET_STATUS   					"%d070151"			//ID:81(0x51) bit0:予約 bit1:0=サーボオフ 1=トルク制御 2=速度制御

#define ASPINA_SET_SERVOOFF_MODE			"%d06031f%02x00"	// ID:31(0x1f) サーボオフモード設定 引数1(0:フリー 1:ショートブレーキ)
#define ASPINA_GET_SERVOOFF_MODE			"%d07011f"			// ID:31(0x1f) サーボオフモード取得 引数1(0:フリー 1:ショートブレーキ)

#define ASPINA_SET_SERVO_MODE				"%d060328%02x00"	// ID:40(0x28) サーボモードの設定 引数1(1:トルク制御 2:速度制御 3:位置制御)
#define ASPINA_GET_SERVO_MODE				"%d070128"			// ID:40(0x28) サーボモードの設定 引数1(1:トルク制御 2:速度制御 3:位置制御)

#define ASPINA_SET_ORDER_MODE				"%d060429%02x%02x00"// ID:41(0x29) 指令モードの設定 引数1(1:トルク制御 2:速度制御 3:位置制御) 引数2(0～3:ダイレクト,アナログ,ポイント,パルス)
#define ASPINA_SET_ORDER_TORQUE_MODE		"%d06042901%02x00"	// ID:41(0x29) トルク制御時の指令モードの設定 引数1(0～3:ダイレクト,アナログ,ポイント,パルス)
#define ASPINA_SET_ORDER_SPEED_MODE		"%d06042902%02x00"	// ID:41(0x29) 速度制御時の指令モードの設定 引数1(0～3:ダイレクト,アナログ,ポイント,パルス)

#define ASPINA_GET_ORDER_MODE				"%d070129%02x"		// ID:41(0x29) 指令モードの取得 引数1(1:トルク制御 2:速度制御 3:位置制御)
#define ASPINA_GET_ORDER_TORQUE_MODE		"%d07012901"		// ID:41(0x29) 指令モードの設定
#define ASPINA_GET_ORDER_SPEED_MODE		"%d07012902"		// ID:41(0x29) 指令モードの設定

#define ASPINA_SET_ORDER_TORQUE			"%d06092b%08x%08x"	// ID:43(0x2b) トルク制御ダイレクト指令 引数1(トルク制御指令値) 引数2(速度制限値)
#define ASPINA_GET_ORDER_TORQUE			"%d07012b"			// ID:43(0x2b) トルク制御ダイレクト値取得

#define ASPINA_SET_ORDER_SPEED				"%d06052c%08x"		// ID:44(0x2c) speed制御ダイレクト指令 引数1(トルク制御指令値) 引数2(速度制限値)
#define ASPINA_GET_ORDER_SPEED				"%d07012c"			// ID:44(0x2c) speed制御ダイレクト値取得





#define ASPINA_SET_LIMITTER					"%d06092A%08x%08x"	//todo 公開するか検討
#define ASPINA_GET_LIMITTER					"%d07012A"			//todo 公開するか検討






/**
 * パラメータID
 */
#define ASPINA_PID_VERSION			(1u)
#define ASPINA_PID_CURRENT_ALARM	(0x50u)
#define ASPINA_PID_CURRENT_SPEED	(0x2fu)
#define ASPINA_PID_CURRENT_TORQUE	(0x2eu)
#define ASPINA_PID_STATUS			(0x51u)
#define ASPINA_PID_SERVOOFF_MODE	(0x1fu)
#define ASPINA_PID_SERVO_MODE		(0x28u)
#define ASPINA_PID_ORDER_MODE		(0x29u)
#define ASPINA_PID_ORDER_TORQUE	(0x2bu)
#define ASPINA_PID_ORDER_SPEED		(0x2cu)
#define ASPINA_PID_LIMITTER			(0x2au)


/**
 * 設定
 */
#define ASPINA_SERVO_OFF			(0)
#define ASPINA_SERVO_ON			(1)

#define ASPINA_SERVOMODE_TORQUE	(1)
#define ASPINA_SERVOMODE_SPEED		(2)

#define ASPINA_ORDERMODE_DIRECT	(0)
#define ASPINA_ORDERMODE_ANALOGUE	(1)
#define ASPINA_ORDERMODE_POINT		(2)
#define ASPINA_ORDERMODE_PULSE		(3)


/**
 * @class AspinaCommands
 * @brief ASPINAノード送受信を簡略化するためのクラス
 */
class AspinaCommands {
private:
	char buff[100];
	ros::Publisher* serialPub;
	std_msgs::String serialMsg;
	bool mReceived;

	// ドライバの応答を保存
	uint16_t mNDVersion[ID_MAX];			//ソフトウェアバージョン
	uint16_t mNDAlarm[ID_MAX];			//現在アラーム
	int32_t mNDCurSpeed[ID_MAX];			//現在速度
	int32_t mNDCurTorque[ID_MAX];		//現在電流
	uint16_t mNDStatus[ID_MAX];			//ドライバ状態
	uint16_t mNDServoMode[ID_MAX];		//サーボモード
	uint16_t mNDOrderMode[ID_MAX][4];	//指令方法
	int32_t mNDOrderTorque[ID_MAX];		//指令電流
	int32_t mNDOrderSpeed[ID_MAX];		//指令速度
	int32_t mNDLimitSpeed[ID_MAX];		//速度上限
	int32_t mNDLimitCurrent[ID_MAX];		//電流上限
	uint16_t mNDServoOffMode[ID_MAX];	//サーボオフモード
	int32_t mNDSystemLimitSpeed[ID_MAX];	//システム速度上限
	int32_t mNDSystemLimitCurrent[ID_MAX];	//システム電流上限

private:
	int32_t asciiToHex(char asciiChar);
	
public:
	AspinaCommands(void);
	void SetPublisherHandle(ros::Publisher* set);
	void ClearReceiveFlag(void);
	void Callback(const char* str);
	uint16_t GetVersion(uint8_t id);
	uint16_t GetAlarm(uint8_t id);
	int32_t GetCurSpeed(uint8_t id);
	int32_t GetCurTorque(uint8_t id);
	uint16_t IsServoOn(uint8_t id);
	uint16_t GetServoMode(uint8_t id);
	uint16_t GetOrderMode(uint8_t servoMode, uint8_t id);
	uint16_t GetOrderSpeedMode(uint8_t id);
	int32_t GetOrderTorque(uint8_t id);
	int32_t GetOrderSpeed(uint8_t id);
	int32_t GetLimitSpeed(uint8_t id);
	uint16_t GetServoOffMode(uint8_t id);
	int32_t GetLimitCurrent(uint8_t id);
	int32_t GetSytemLimitSpeed(uint8_t id);
	int32_t GetSytemLimitCurrent(uint8_t id);
	char* CreateCommand(const char* str, uint8_t id);
	char* CreateCommand(const char* str, int32_t x, uint8_t id);
	char* CreateCommand(const char* str, int32_t x, int32_t y, uint8_t id);
	void PublishString(const char* str, uint8_t id);
	void PublishCommand(const char* str, uint8_t id);
 	void PublishCommand(const char* str, int32_t x, uint8_t id);
	void PublishCommand(const char* str, int32_t x, int32_t y, uint8_t id);
	bool IsReceived(void);
	void WaitReceive(void);
	void Handshake(uint8_t id);
};

